from bgez.asyn.task_manager import TaskManager
from bgez.core.dispatcher import Dispatcher

__all__ = [
    'AsynManager',
]

class AsynManager(Dispatcher):
    '''Manages asynchronous modules.'''

    def add_task_manager(self, pool_size=5, max_tasks_queued=10):
        '''
        Same as instantiating a new TaskManager on its own, but here the AsynManager keeps tracks
        of it. The later behaviour allows to call or close all task managers at once.
        '''
        task_manager = TaskManager(pool_size, max_tasks_queued)
        task_manager.on_close.subscribe(self.unuse, once=True)
        self.use(task_manager)
        return task_manager

    def close_all(self):
        '''Closes all child managers.'''
        for handler in self.handlers:
            if hasattr(handler, "close"):
                handler.close()

    def dispatch(self, *args, **kargs):
        return super().dispatch()
