import sys

from collections import OrderedDict
from importlib import import_module

from . import PythonComponent

__all__ = [
    'Launcher'
]

MODULE = 'Main Python Module'

class Launcher(PythonComponent):

    args = OrderedDict([
        (MODULE, ''),
    ])

    def start(self, args):
        from bge import logic # pylint: disable=E0401
        self.error = None

        self.before = None
        self.now = None

        try: # framework development safe guard
            from bgez import blender, loop
            self.game = blender.game
        except Exception as e:
            self.error = e
            logic.endGame()
            raise e

        loop.set(blender.loop)

        module_id = args.get(MODULE)
        try: # runtime user setup
            self.module = import_module(module_id)
        except Exception as e:
            self.error = e
            logic.endGame()
            raise e

        # notify the user that the game started
        self.game._emit_start()

    def update(self):
        if self.error is not None:
            return

        from bge import logic # pylint: disable=E0401
        try: # stop on handling errors
            self.game._step()
        except Exception as e:
            self.game._throw_error(e)
            self.game.quit()
            raise e
