from typing import Callable

from functools import wraps

__all__ = [
    'Abstract', 'abstractmethod',
]

def abstractmethod(function: Callable) -> Callable:
    '''
    Decorate functions with this in order to mark them as abstract.
    An abstract method needs to be implemented by child classes.
    '''

    @wraps(function)
    def notimplemented(*args, **kargs):
        docstring = getattr(function, '__doc__', None)
        if docstring is None: docstring = 'n/a'
        raise NotImplementedError(f'abstract\ndocstring: {docstring}')

    return notimplemented

class Abstract:
    '''
    Inherit from this class to define abstract/base classes.
    '''

    def __init__(self):
        raise NotImplementedError('Abstract classes should not be instanciated')
