from bgez.core.input import InputType

__all__ = [
    'AxisType',
]

class AxisType(InputType):
    '''
    Float values between -1 and 1.
    '''

    @staticmethod
    def _normalize(values):
        return (max(min(value, 1), -1) for value in values)

    @classmethod
    def union(cls, values) -> float:
        '''
        Returns the furthest value from zero from `values`.\n
        This represents the case where multiple axis contribute to one action.
        '''
        return max(cls._normalize(values), key=abs)

    @classmethod
    def intersection(cls, values) -> float:
        '''
        Returns the closest value from zero from `values`.\n
        Not sure what this represents yet, but if you were to compare two
        joysticks this way, you would have to push them in the same direction
        to see any effect.
        '''
        return min(cls._normalize(values), key=abs)

    @staticmethod
    def threshold(value):
        '''
        Returns `None` if `value` is below `threshold`, returns `value` otherwise.
        '''
        def function(threshold: float, fallback = None):
            if abs(value) >= threshold:
                return value
            return fallback
        return function
