from bgez.core.input import InputType

__all__ = [
    'ButtonType',
]

class ButtonType(InputType):

    RELEASED = 0
    RELEASING = 1
    PRESSING = 2
    PRESSED = 3

    STATES = (RELEASED, RELEASING, PRESSING, PRESSED)

    @classmethod
    def union(cls, values):
        '''
        Returns the highest state in `states`.\n
        This represents the case where multiple keys contribute to one action.
        (you can press either one or the other)
        '''
        return max(values)

    @classmethod
    def intersection(cls, values):
        '''
        Returns the lowest state in `states`.\n
        This represents the case where multiple keys represent one action.
        (you have to press every key)
        '''
        return min(values)

    @classmethod
    def released(cls, value):
        return value == cls.RELEASED

    @classmethod
    def releasing(cls, value):
        return value == cls.RELEASING

    @classmethod
    def pressing(cls, value):
        return value == cls.PRESSING

    @classmethod
    def pressed(cls, value):
        return value == cls.PRESSED

    @classmethod
    def active(cls, value):
        return value == cls.PRESSED or value == cls.PRESSING

    @classmethod
    def inactive(cls, value):
        return value == cls.RELEASED or value == cls.RELEASING
