from weakref import finalize
from typing import Callable

from bgez.core.events import Subject

__all__ = [
    'Sentinel',
]

class Sentinel:
    '''Object that monitors its own garbage collection'''

    def __init__(self, callback: Callable = None):
        self._subject = subject = Subject()
        if callback: self.on_gc(callback)
        self._finalizer = finalize(self, subject.notify)

    def on_garbage_collection(self, *args, **kargs):
        return self._subject.subscribe(*args, **kargs)
    on_gc = on_garbage_collection

    def unsubscribe(self, callback: Callable):
        return self._subject.unsubscribe(callback)
