from typing import Callable, Tuple, Optional

__all__ = [
    'Timer',
    'TimeoutTimer',
    'IntervalTimer',
]

class Timer:

    def __init__(self, deadline, callback, args, kargs):
        self.parameters = callback, args, kargs
        self.deadline = deadline

    def __call__(self) -> Tuple[Optional[float], Optional[Callable]]:
        callback, args, kargs = self.parameters
        callback(*args, **kargs)
        return None, None

    def __lt__(self, other):
        return False

class TimeoutTimer(Timer):

    def __init__(self, clock: Callable, timeout, callback, args, kargs):
        super().__init__(max(timeout, 0) + clock(), callback, args, kargs)

class IntervalTimer(TimeoutTimer):

    def __init__(self, clock: Callable, interval, callback, args, kargs):
        super().__init__(clock, interval, callback, args, kargs)
        self.interval = interval

    def __call__(self):
        super().__call__()
        self.deadline += self.interval
        return self.deadline, self
