from bgez.core.input import InputType, ButtonType, AxisType, Input
from bgez.core.abstract import abstractmethod

from bge import logic # pylint: disable=E0401

__all__ = [
    'KX_ButtonInput', 'Keyboard', 'Mouse',
    'Joystick', 'JoystickButton', 'JoystickAxis',
]

BGE_STATE_MAP = {
    logic.KX_INPUT_NONE: ButtonType.RELEASED,
    logic.KX_INPUT_JUST_RELEASED: ButtonType.RELEASING,
    logic.KX_INPUT_JUST_ACTIVATED: ButtonType.PRESSING,
    logic.KX_INPUT_ACTIVE: ButtonType.PRESSED,
}

class KX_ButtonInput(Input):
    TYPE = ButtonType

    def __init__(self, keycode):
        self.keycode = keycode

    @abstractmethod
    def _get_state(self):
        '''Returns the state of the key'''

    @property
    def value(self):
        return BGE_STATE_MAP[self._get_state()]

class Keyboard(KX_ButtonInput):
    def _get_state(self):
        return logic.keyboard.events[self.keycode]

class Mouse(KX_ButtonInput):
    def _get_state(self):
        return logic.mouse.events[self.keycode]

class Joystick:

    def __init__(self, index: int):
        self.index = index

    def get(self):
        index = self.index
        joysticks = logic.joysticks
        if index >= len(joysticks):
            return None
        return joysticks[index]

class JoystickButton(Input, Joystick):
    TYPE = ButtonType

    def __init__(self, joystick: int, button: int):
        Joystick.__init__(self, joystick)
        self.button = button

    @property
    def value(self):
        button = self.button
        joystick = self.get()
        if joystick is None:
            return None
        if button in joystick.activeButtons:
            return ButtonType.PRESSED
        return ButtonType.RELEASED

class JoystickAxis(Input, Joystick):
    TYPE = AxisType

    def __init__(self, joystick: int, axis: int):
        Joystick.__init__(self, joystick)
        self.axis = axis

    @property
    def value(self):
        axis = self.axis
        joystick = self.get()
        if (joystick is None) or (axis >= joystick.numAxis):
            return None
        return joystick.axisValues[axis]
