from bgez.core.events import EventEmitter

__all__ = [
    'Stage',
]

class Stage:
    '''
    A stage is an abstraction layer for your logic to be run inside a scene.
    Stages will replace each others within the same scene, eventually.
    '''

    logic = None
    pre_draw_setup = None
    pre_draw = None
    post_draw = None

    def __init__(self):
        self.__previous = None
        self.__scene = None

    def __remove(self, scene):
        self.unload()

    @property
    def previous(self):
        return self.__previous

    @property
    def scene(self):
        return self.__scene

    def attach(self, scene: EventEmitter, previous):
        self.__previous = previous
        self.__scene = scene
        if self.logic:
            scene.on('logic', self.logic)
        if self.pre_draw_setup:
            scene.on('pre_draw_setup', self.pre_draw_setup)
        if self.pre_draw:
            scene.on('pre_draw', self.pre_draw)
        if self.post_draw:
            scene.on('post_draw', self.post_draw)
        scene.on('remove', self.__remove)
        self.load()

    def detach(self):
        scene = self.__scene
        scene.remove('logic', self.logic)
        scene.remove('pre_draw_setup', self.pre_draw_setup)
        scene.remove('pre_draw', self.pre_draw)
        scene.remove('post_draw', self.post_draw)
        scene.remove('remove', self.__remove)
        self.unload()
        self.__scene = None

    def load(self):
        '''Loads the stage and its resources'''

    def unload(self):
        '''Unloads the stage and its resources'''
