import sys

from os import path
from glob import iglob
from platform import system

__all__ = [
    'is_windows',
    'is_linux',
    'is_mac',
    'is_blender',
    'is_upbge',
    'is_game',
]

is_windows = system() == 'Windows'
is_linux = system() == 'Linux'
is_mac = system() == 'Darwin'

def get_python_exe():
    if is_windows or is_linux:
        blender_path = path.dirname(sys.executable)
        python_path = path.join(blender_path, '*', 'python', 'bin', 'python*')
        return next(iglob(python_path), sys.executable)
    return sys.executable
python = get_python_exe()

try:
    from bpy import app
    is_upbge = getattr(app, 'upbge_version', None) is not None
    is_blender = is_upbge or getattr(app, 'version', None) is not None
except ImportError:
    is_blender = False
    is_upbge = False

try:
    from bge import app, logic, types, texture, events, constraints
    del app, logic, types, texture, events, constraints
    is_game = True
except ImportError:
    is_game = False
